@@ -1,9 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-Authors: Tim Hessels
-Module: Collect
 
-Description:
 This module contains scripts used to download Level 1 data (data directly from web).
 
 Products                      Dates                             Password
@@ -25,8 +22,8 @@ help(Collect)
 dir(Collect)
 """
 
-from pywapor.collect import CHIRPS, MOD11, MYD11, MCD43, MOD13, MYD13, MSGCPP, MERRA, Globcover, GEOS, SRTM, WAPOR, PROBAV, get_pw_un, setup_dl_accounts#, Landsat
+from pywapor.collect import CHIRPS, MOD11, MYD11, MCD43, MOD13, MYD13, MERRA2, Globcover, GEOS5, SRTM, WAPOR, PROBAV, STATICS, get_pw_un, setup_dl_accounts, Landsat, sideloader, downloader
 
-__all__ = ['CHIRPS', 'MOD11', 'MYD11', 'MCD43', 'MOD13', 'MYD13', 'MSGCPP', 'MERRA', 'SRTM', 'Globcover', 'GEOS', 'WAPOR' , 'PROBAV', 'get_pw_un', 'setup_dl_accounts']#, 'Landsat']
+__all__ = ['CHIRPS', 'MOD11', 'MYD11', 'MCD43', 'MOD13', 'MYD13', 'MERRA2', 'SRTM', 'Globcover', 'GEOS5', 'WAPOR' , 'PROBAV', 'STATICS', 'get_pw_un', 'setup_dl_accounts', 'Landsat', 'sideloader', 'downloader']
 
 __version__ = '0.1'
